{----------------------------------------------------------}
{                                                          }
{              Copyrighted (C) John H. Gohde, 1991         }
{                    All Rights Reserved                   }
{                                                          }
{           FreeWare licensed for NON-Commercial use.      }
{                                                          }
{----------------------------------------------------------}

{$A+,B-,D-,E-,F+,G-,I-,L-,N-,O-,R-,S-,V-,X-}

Unit BIT;

INTERFACE

Type
      Str16         = string[16];

      FlagPos       = 1..16;

      NibblePos     = 1..4;

      Flags         = word; { True/False }

      SplitWord = record
                    Hi,
                    Lo: Byte;
                  end;

Const
  BITVer       = '1.00';

      FUNCTION  FlagsOn    (F : Flags; NUM : FlagPos) : Boolean;
      Function  GetDoubleByte(VAR DB : word; NUM : NibblePos): byte;
      FUNCTION  GetHiNibble  (DN : byte) : byte;
      FUNCTION  GetLoNibble  (DN : byte) : byte;
      Procedure SetDoubleByte(VAR DB : word; NUM : NibblePos; Value: Byte);
      PROCEDURE SetFlags   (VAR F : Flags; NUM : FlagPos; SW: Boolean);
      Function  SetHiNibble(DN,Hi: byte): byte;
      Function  SetLoNibble(DN,Lo: byte): byte;
      FUNCTION  SwapNibbles  ( DN : byte) : byte;
      PROCEDURE ToggleFlags(VAR F : Flags; NUM : FlagPos);
      Function  Flags2Str           ( F: Flags): str16;


{*********************************************************************}

IMPLEMENTATION

{---------------------------------------------------------- BIT MAPPING -}


FUNCTION FlagsOn(F : Flags; NUM : FlagPos) : Boolean; Assembler;
ASM
      INC   NUM
      MOV   CL,   NUM
      MOV   BX,   1
      SHL   BX,   CL
      MOV   AL,   0
      TEST  F,    BX
      JZ    @NO
      MOV   AX,   1
      JMP   @EXIT
@NO:  MOV   AX,   0
@EXIT:
END;

Function GetHiNibble(DN: byte): byte; assembler;
asm
    MOV   AL,   DN
    MOV   CL,   4
    SHR   AL,   CL
end;

Function GetLoNibble(DN: byte): byte; assembler;
asm
    MOV   AL,   DN
    AND   AL,   0FH
end;

Procedure SetDoubleByte(VAR DB : word; NUM : NibblePos; Value: Byte);
var W: SplitWord;
begin
  W:=SplitWord(DB);
  Case NUM of
  1: W.Hi:=SetHiNibble(W.Hi,Value);
  2: W.Hi:=SetLoNibble(W.Hi,Value);
  3: W.Lo:=SetHiNibble(W.Hi,Value);
  4: W.Lo:=SetLoNibble(W.Hi,Value);
  end;
  DB:=word(W);
end;

Function GetDoubleByte(VAR DB : word; NUM : NibblePos): byte;
var W: SplitWord;
    B: BYTE;
begin
  W:=SplitWord(DB);
  Case NUM of
  1: B:=GetHiNibble(W.Hi);
  2: B:=GetLoNibble(W.Hi);
  3: B:=GetHiNibble(W.Lo);
  4: B:=GetLoNibble(W.Lo);
  end;
  DB:=B;
end;

PROCEDURE SetFlags(VAR F : Flags; NUM : FlagPos; SW: Boolean); Assembler;
ASM
      MOV   AL,   SW
      INC   NUM
      MOV   CL,   NUM
      MOV   BX,   1
      SHL   BX,   CL
      LES   DI,   F
      CMP   AL,   1
      JNZ   @NO
      OR    ES:[DI],  BX
      JMP   @EXIT
@NO:  AND ES:[DI], BX
@EXIT:
END;

Function SetHiNibble(DN,Hi: byte): byte;
begin
  SetHiNibble:=GetLoNibble(DN)+(Hi shl 4);
end;

Function SetLoNibble(DN,Lo: byte): byte;
begin
  SetLoNibble:=Lo+(GetHiNibble(DN) shl 4);
end;

FUNCTION SwapNibbles(DN : byte) : byte; Assembler;
ASM
    MOV AL, DN
    MOV AH, AL
    MOV CL, 4
    SHL AL, CL
    SHR AH, CL
    ADD AL, AH
END;

PROCEDURE ToggleFlags(VAR F : Flags; NUM : FlagPos); Assembler;
ASM
      INC   NUM
      MOV   CL,   NUM
      MOV   BX,   1
      SHL   BX,   CL
      LES   DI,   F
      XOR   ES:[DI],  BX
END;

Function Flags2Str(F: Flags):str16;
var S: str16;
    B: BYTE;
begin
  s:='';
  for B:=1 to 16 do
    If FlagsOn(F,B) then S:=S+'T' else S:=S+'F';
  Flags2Str := S;
end;

{----------------------------------------------------------}
{                                                          }
{              Copyrighted (C) John H. Gohde, 1991         }
{                    All Rights Reserved                   }
{                                                          }
{           FreeWare licensed for NON-Commercial use.      }
{                                                          }
{----------------------------------------------------------}

END.
